// **************************************************************************
// *                                                                        *
// *   NeoPixel Ring Clock Alternate Display                                                  *
// *   by John Guarnero                                 	            *
// *   http://guarnero.com                                   	            *
// *   04/19/2016                                                           *
// **************************************************************************

#include "Wire.h"
#include <RTClib.h>
#include <Adafruit_NeoPixel.h>
#include "pitches.h"  // copy the pitches.h file to the directory where your Arduino project resides and then restart Arduino software

#define DS1307_ADDRESS 0x68 //default address of the 1307 RTC module
#define BRIGHTNESS 12 // define max brightness - higher number is brighter.  If the number is too high it will exceed current draw of Arduino


// setup for the westminster hour and half hour chime
int melody[] = 
{
NOTE_E3, NOTE_GS3, NOTE_FS3, NOTE_B2, 0,
NOTE_E3, NOTE_FS3, NOTE_GS3, NOTE_E3, 0,
};
int noteDurations[] = 
{
4, 4, 4, 2.5, 4,
4, 4, 4, 2.5, 4,
};


int chime_active = 1; //initialize variable used for indicating if chime is on or off
int Oneshot_Clear = 0;
byte one = 0x01; //variable used to point to time setting for minutes in DS1307
byte two = 0x02; //variable used to point to time setting for minutes in DS1307

byte hourval, minuteval, secondval; // holds the actual time
byte pixelColorRed, pixelColorGreen, pixelColorBlue; // holds color values for NeoPixels

byte hourval_set;   // holds the hour set time
byte minuteval_set; // holds the minute set time


RTC_DS1307 RTC; // Establish clock object
DateTime Clock; // Holds current clock time

Adafruit_NeoPixel strip = Adafruit_NeoPixel(60, 6, NEO_GRB + NEO_KHZ800); // strip object for 60 neopixels controlled on pin 6


void setup() {
  
  pinMode(5, INPUT_PULLUP); // declare pin 5 to be an input with internal pullups (Red Wire - Set Time Pushbutton):
  pinMode(2, INPUT_PULLUP); // declare pin 2 to be an input with internal pullups (Blue Wire - Increment Minute Pushbutton):
  pinMode(3, INPUT_PULLUP); // declare pin 3 to be an input with internal pullups (Green Wire - Increment Hour Pushbutton):
  pinMode(4, INPUT_PULLUP); // declare pin 4 to be an input with internal pullups (White Wire - Chime On/Off Pushbutton):
  pinMode(8, OUTPUT);       // declare pin 8 to be an output (DHD Dial Crystal LED):
  pinMode(6, OUTPUT);       // declare pin 6 to be an output (NeoPixel Control Pin):

  Wire.begin();
  RTC.begin();              // begin clock
  Serial.begin(9600);       // define serial for debug of program
  
//     RTC.adjust(DateTime(__DATE__, __TIME__));  // Run this to set the time to the PC Time on Arduino power up - enable for debug and troubleshooting only

}

void loop() {

skipchime:  

  int button_Set_Time = digitalRead(5);    // Read the Set Time Pushbutton
  int button_Minute_Inc = digitalRead(2);  // Read the Increment Minute Pushbutton
  int button_Hour_Inc = digitalRead(3);    // Read the Increment Hour Pushbutton
  int button_Set_Chime = digitalRead(4);   // Read the Chime On/Off Pushbutton
  
  uint32_t red = strip.Color(255, 0, 0);   //define NeoPixel Red color
  uint32_t green = strip.Color(0, 255, 0); //define NeoPixel Green color
  uint32_t blue = strip.Color(0, 0, 255);  //define NeoPixel Blue color
  uint32_t off = strip.Color(0, 0, 0);     //define NeoPixel No color

  // Get the current time from the real time clock module
  Clock = RTC.now();           // get the RTC time

  secondval = Clock.second();  // get seconds
  minuteval = Clock.minute();  // get minutes
  hourval = Clock.hour();      // get hours

 // This is here for serial debug as needed
 // Serial.print("set time: "); Serial.print(button_Set_Time); Serial.print('\n');
 //  Serial.print("hour: "); Serial.print(button_Hour_Inc); Serial.print('\n');
 //  Serial.print("minute: "); Serial.print(button_Minute_Inc); Serial.print('\n');
 //  Serial.print("chime: "); Serial.print(button_Set_Chime); Serial.print('\n');
 //  Serial.print('\n');

  if (hourval > 11) hourval -= 12;             // if hourval > 11 then hourval -= hourval - 12  this just makes it 12 hour clock
  hourval = (hourval * 60 + minuteval) / 12;   //determine the red dot to illuminate for example 3:30 is  (3 * 60 + 30) / 12 = 17.5 so pixel is 17 is illuminated since Arduino truncates remainder 


 //Setup initial colors for the hours, minutes, and seconds
 
  strip.setPixelColor(hourval, red);

  strip.setBrightness(BRIGHTNESS); // set brightness level defined at top of program


 if (secondval == 1)
 {
 Oneshot_Clear = 0;  
 }

  //Check if the seconds just went to 0.  If they did, turn off the clock seconds LEDs and start again
  if (secondval == 0 && Oneshot_Clear == 0)
  {
    Oneshot_Clear = 1;  
    strip.setPixelColor(0, off);
    strip.setPixelColor(1, off);
    strip.setPixelColor(2, off);
    strip.setPixelColor(3, off);
    strip.setPixelColor(4, off);
    strip.setPixelColor(5, off);
    strip.setPixelColor(6, off);
    strip.setPixelColor(7, off);
    strip.setPixelColor(8, off);
    strip.setPixelColor(9, off);
    strip.setPixelColor(10, off);
    strip.setPixelColor(11, off);
    strip.setPixelColor(12, off);
    strip.setPixelColor(13, off);
    strip.setPixelColor(14, off);
    strip.setPixelColor(15, off);
    strip.setPixelColor(16, off);
    strip.setPixelColor(17, off);
    strip.setPixelColor(18, off);
    strip.setPixelColor(19, off);
    strip.setPixelColor(20, off);
    strip.setPixelColor(21, off);
    strip.setPixelColor(22, off);
    strip.setPixelColor(23, off);
    strip.setPixelColor(24, off);
    strip.setPixelColor(25, off);
    strip.setPixelColor(26, off);
    strip.setPixelColor(27, off);
    strip.setPixelColor(28, off);
    strip.setPixelColor(29, off);
    strip.setPixelColor(30, off);
    strip.setPixelColor(31, off);
    strip.setPixelColor(32, off);
    strip.setPixelColor(33, off);
    strip.setPixelColor(34, off);
    strip.setPixelColor(35, off);
    strip.setPixelColor(36, off);
    strip.setPixelColor(37, off);
    strip.setPixelColor(38, off);
    strip.setPixelColor(39, off);
    strip.setPixelColor(40, off);
    strip.setPixelColor(41, off);
    strip.setPixelColor(42, off);
    strip.setPixelColor(43, off);
    strip.setPixelColor(44, off);
    strip.setPixelColor(45, off);
    strip.setPixelColor(46, off);
    strip.setPixelColor(47, off);
    strip.setPixelColor(48, off);
    strip.setPixelColor(49, off);
    strip.setPixelColor(50, off);
    strip.setPixelColor(51, off);
    strip.setPixelColor(52, off);
    strip.setPixelColor(53, off);
    strip.setPixelColor(54, off);
    strip.setPixelColor(55, off);
    strip.setPixelColor(56, off);
    strip.setPixelColor(57, off);
    strip.setPixelColor(58, off);
    strip.setPixelColor(59, off);
  }
  {
    
  for (int i_min = 0; i_min <= minuteval; i_min++)
  {
    
    strip.setPixelColor(i_min, green);
//    strip.setPixelColor(secondval, blue);  //uncomment if you want seconds to appear
    strip.setPixelColor(hourval, red);
    strip.show();
    delay(1);
  }
  }
  
//Play Westminister on the hour if chime is set and also turn on the LED
  if (minuteval == 0 && secondval == 0 && chime_active == 1) 
  {
    digitalWrite(8, HIGH);                      //turn on LED 
    // iterate over the notes of the melody:
  for (int thisNote = 0; thisNote < 9; thisNote++) {
    // to calculate the note duration, take one second 
    // divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000/noteDurations[thisNote];
    tone(10, melody[thisNote],noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30; // to distinguish the notes, set a minimum time between them.
    delay(pauseBetweenNotes);
    noTone(10); // stop the tone playing:
 
  }
  }

//turn the LED off at 1 minute since it is turned on when the hour changes if the time is set
 if (minuteval == 1) 
  {
    digitalWrite(8, LOW); 
  }
  

//Play last half Westminister on the hour if chime set and turn on the LED
  if (minuteval == 30 && secondval == 0 && chime_active == 1) 
  {
    digitalWrite(8, HIGH);                //turn on the LED
   // iterate over the notes of the melody:
  for (int thisNote = 5; thisNote < 9; thisNote++) {
    // to calculate the note duration, take one second 
    // divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000/noteDurations[thisNote];
    tone(10, melody[thisNote],noteDuration);

    // to distinguish the notes, set a minimum time between them.
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    noTone(10);  // stop the tone playing:
  }
  }

//turn the LED off at 31 minutes since it is turned on at 30 minutes if chime is set
if (minuteval == 31) 
  {
    digitalWrite(8, LOW);
  }
 

//Check the pushbuttons

//check to see if the set time and change hours button is pressed
  if (button_Set_Time == LOW && button_Hour_Inc == LOW && button_Minute_Inc == HIGH && button_Set_Chime == HIGH)
  {
    clearPixels();
    change_hours();
    delay(300);
  }
  
//check to see if the set time and change minutes button is pressed
  if (button_Set_Time == LOW && button_Hour_Inc == HIGH && button_Minute_Inc == LOW && button_Set_Chime == HIGH)
  {
    clearPixels();
    change_minutes();
    delay(150);
  }

//check to see if the Set Chime button is pressed and the chime is not currently active
  if (button_Set_Time == HIGH && button_Hour_Inc == HIGH && button_Minute_Inc == HIGH && button_Set_Chime == LOW && chime_active == 0)
  {
  delay (500);
  chime_active = 1;
  tone(10,NOTE_G6,125);
  delay(130);
  tone(10,NOTE_C7,125);
  delay(130);
  tone(10,NOTE_G7,125);
  delay(125);
  noTone(10);
  delay (1000);
  goto skipchime; 
  }


//check to see if the Set Chime button is pressed and the chime is currently active
  if (button_Set_Time == HIGH && button_Hour_Inc == HIGH && button_Minute_Inc == HIGH && button_Set_Chime == LOW && chime_active == 1)
  {
  delay (500);
  chime_active = 0;
  tone(10,NOTE_A3,125);
  delay(130);
  tone(10,NOTE_A2,200);
  delay(205);
  tone(10,NOTE_A1,150);
  delay(155);
  noTone(10);
  delay (1000);
  goto skipchime;
  }

 
}

void change_hours() //Change the clock hours
{ 
  if (hourval_set < 24)
  {
    hourval_set = hourval_set + 1;
    delay (100);
  }

  if (hourval_set >= 24)
  {
    hourval_set = 0;
  }
  setDateTime_hours();
}



void change_minutes() //Change the clock minutes
{ 
  if (minuteval_set < 60)
  {
    minuteval_set = minuteval_set + 1;
    delay (50);
  }

  if (minuteval_set >= 60)
  {
    minuteval_set = 0;
  }
  setDateTime_minutes();
}
 

void setDateTime_hours() //Change the clock hours in the DS1307
{
  Wire.beginTransmission(DS1307_ADDRESS);
  Wire.write(two); //stop Oscillator
  Wire.write(decToBcd(hourval_set));
  Wire.endTransmission();
}


void setDateTime_minutes() //Change the clock minutes in the DS1307
{
  Wire.beginTransmission(DS1307_ADDRESS);
  Wire.write(one); //stop Oscillator
  Wire.write(decToBcd(minuteval_set));
  Wire.endTransmission();
}


void clearPixels() { //Turn off the NeoPixels - This happens each time the seconds change to zero.
    uint32_t off = strip.Color(0, 0, 0);      //define NeoPixel No color
    uint32_t blue = strip.Color(0, 0, 255);   //define NeoPixel Blue color
    strip.setPixelColor(0, off);
    strip.setPixelColor(1, off);
    strip.setPixelColor(2, off);
    strip.setPixelColor(3, off);
    strip.setPixelColor(4, off);
    strip.setPixelColor(5, off);
    strip.setPixelColor(6, off);
    strip.setPixelColor(7, off);
    strip.setPixelColor(8, off);
    strip.setPixelColor(9, off);
    strip.setPixelColor(10, off);
    strip.setPixelColor(11, off);
    strip.setPixelColor(12, off);
    strip.setPixelColor(13, off);
    strip.setPixelColor(14, off);
    strip.setPixelColor(15, off);
    strip.setPixelColor(16, off);
    strip.setPixelColor(17, off);
    strip.setPixelColor(18, off);
    strip.setPixelColor(19, off);
    strip.setPixelColor(20, off);
    strip.setPixelColor(21, off);
    strip.setPixelColor(22, off);
    strip.setPixelColor(23, off);
    strip.setPixelColor(24, off);
    strip.setPixelColor(25, off);
    strip.setPixelColor(26, off);
    strip.setPixelColor(27, off);
    strip.setPixelColor(28, off);
    strip.setPixelColor(29, off);
    strip.setPixelColor(30, off);
    strip.setPixelColor(31, off);
    strip.setPixelColor(32, off);
    strip.setPixelColor(33, off);
    strip.setPixelColor(34, off);
    strip.setPixelColor(35, off);
    strip.setPixelColor(36, off);
    strip.setPixelColor(37, off);
    strip.setPixelColor(38, off);
    strip.setPixelColor(39, off);
    strip.setPixelColor(40, off);
    strip.setPixelColor(41, off);
    strip.setPixelColor(42, off);
    strip.setPixelColor(43, off);
    strip.setPixelColor(44, off);
    strip.setPixelColor(45, off);
    strip.setPixelColor(46, off);
    strip.setPixelColor(47, off);
    strip.setPixelColor(48, off);
    strip.setPixelColor(49, off);
    strip.setPixelColor(50, off);
    strip.setPixelColor(51, off);
    strip.setPixelColor(52, off);
    strip.setPixelColor(53, off);
    strip.setPixelColor(54, off);
    strip.setPixelColor(55, off);
    strip.setPixelColor(56, off);
    strip.setPixelColor(57, off);
    strip.setPixelColor(58, off);
    strip.setPixelColor(59, off);
    strip.show();
    delay(10);
}

byte decToBcd(byte val) { // Convert normal decimal numbers to binary coded decimal - used for DS1307
   return ( (val / 10 * 16) + (val % 10) );
}

byte bcdToDec(byte val)  { // Convert binary coded decimal to normal decimal numbers - used for DS1307
   return ( (val / 16 * 10) + (val % 16) );
}
